
( function()
{
    // Load the plugin script and CSS files.
    var path = "/ts/screens/";
    if ( AXES.isTS2Engine )
    {
        AXES.Utils.LazyLoader.load( path + "jquery.keypad.css", null, null, null );
        AXES.Utils.LazyLoader.load( path + "jquery.keypad.min.js", null, null, null );
    } 
    else 
    {
        AxUtil.includeStyleSheet( path + "jquery.keypad.css" );
        AxUtil.includeScript( path + "jquery.keypad.min.js" );
    }
})();

( function()
{
    var extension = {
        name   : "axKeypad", /* Programmatic class name of this extension. It's not visible to end user */
        group  : "vis",
        type   : "element", /* Type of object this extension is applicable. Can be either "element" or "screen" */
        display: TEXT.dev( "axKeypad.display" ),
        order  : 5,

        properties : {
            value : {
                type : "String",
                description : TEXT.dev( "axDefaultVisualization.value" )
            },
            style : {
                type : "Style",
                description : TEXT.dev( "axDefaultVisualization.style" )
            },
            maxInputLength : {
                type : "Integer",
                defaultStatic : 0,
                description : TEXT.dev( "axDefaultVisualization.maxInputLength" )
            },
            onFocus : {
                type: "Event", 
                description: "Fires when this element receives the focus", 
                defaultDynamic: "" 
            },
            onBlur : {
                type: "Event", 
                description: "Fires when this element loses focus", 
                defaultDynamic: "" 
            }
        },

        /* This function creates any objects that will not change during the lifetime of 
         * the eXtension. Do not do anything here that might be dependent on the value of
         * an eXtension property. */
        init : function( field, fieldDiv )
        {
            this._field = field;
            this._fieldDiv = fieldDiv;
            this._content = null;

            // Check the field type and create an appropriate HTML element
            switch ( field.type )
            {
                case AXES.Terminal.Field.Types.OUTPUT:
                    this._createTextField();
                    break;

                case AXES.Terminal.Field.Types.INPUT:
                    this._createInputField();
                    break;
            }

            if ( this._content && this._content.length > 0 )
            {
                this.backupStyle( this._content );
                this._fieldDiv.appendChild( this._content[ 0 ] );
            }

            if ( AXES.isTS2Engine )
                this._field.addListener( "AddedToDOM", this._handleAddedToDOM, this );
        },

        _createTextField : function()
        {
            var newEl;
            if ( AXES.isTS2Engine )
            {
                newEl = document.createElement( "span" );
                newEl.className = "ax-terminal-textField";

                this._content = $( newEl ).bind(
                                            "click",
                                            this._field,
                                            function( e )
                                            {
                                                e.data.setCursor();
                                            }
                );
            }
            else
            {
                newEl = document.createElement( "pre" );
                this._content = $( newEl );
            }
            newEl.innerHTML = this._field.getValue();

            this._field.addListener( "ValueChanged", this._handleFieldValueChanged, this );
        },

        _createInputField : function()
        {
            var field = this._field;
            var newEl = document.createElement( "input" );
            var css = {};

            if ( AXES.isTS2Engine )
            {
                newEl.className = "ax-terminal-inputField";
                if ( field.rightAlign )
                {
                    css[ "text-align" ] = "right";
                    css[ "padding-right" ] = "1px";
                }
                if ( field.width )
                {
                    css[ "width" ] = AXES.Layout.characterDimensionsToPixels( { width : field.width }, false, true ).width;
                    if ( field.rightAlign )
                        css[ "width" ] -= 1;
                    css[ "width" ] += "px";
                }
            }
            this._content = $( newEl ).bind( "blur", this, this._handleBlur).bind( "focus", this, this._handleFocus ).css(css);

            if ( !AXES.isTS2Engine )
                // The old layout manager doesn't enforce a blur before actions like a post
                // so send the onchange event to the blur handler
                this._content.bind( "change", this, this._handleBlur );

            if ( this.applyTabIndex )
                this.applyTabIndex( newEl, true );
            if ( field )
            {
                if ( field.len )
                    newEl.setAttribute( "maxlength", field.len );
                if ( field.width )
                    newEl.setAttribute( "size", field.width );
                this._content.val( field.getValue() );
                if ( AXES.isTS2Engine )
                    field.addListener( "SetFocus", this._setFocus, this );
                field.addListener( "ValueChanged", this._handleFieldValueChanged, this );
            }
        },

        /* This method renders the eXtension for display. In runtime mode it is called once immediately
         * after the init method. In design mode it is called by the developer pane every time it feels
         * the eXtension might need redrawing (usually each time a property is changed) */
        render : function( field, fieldDiv )
        {
            this._fieldDiv = fieldDiv;
            var type = ( field ) ? field.type : AXES.Terminal.Field.Types.INPUT;
            var newStyle = this.getPropertyValue( "style" );
            var maxLength = this.getPropertyValue( "maxInputLength" );

            if ( typeof maxLength == "number" && !isNaN( maxLength ) && maxLength > 0 )
                this._content[ 0 ].setAttribute( "maxlength", maxLength );

            this._content.keypad( "destroy" ); //Destroy the keypad so it can be recreated with new settings
            this.applyStyle2( {}, this._content, true ); // Reset style to the value backed up in init

            this._setValue();

            if ( AXES.isTS2Engine )
                // Apply new styles
                this.applyStyle2( newStyle, this._content[ 0 ], true );
            else
            {
                // TS2 does this in the AddedToDOM event handler
                var size = field.getSize();
                this._content[ 0 ].style.width = size.width + "px";

                baseStyle = {
                    width: size.width,
                    height: size.height
                };
                this.applyStyle( this.getPropertyValue( "style" ), this._content[ 0 ], baseStyle );
            }

            var keypad = $.keypad;
            var keypadSettings = {};
            if ( "fr" == TEXT.currentLanguage )
                keypadSettings.layout = [ '123' + keypad.CLOSE, '456' + keypad.CLEAR, '789' + keypad.BACK, ',0' ];
            else
                keypadSettings.layout = [ '123' + keypad.CLOSE, '456' + keypad.CLEAR, '789' + keypad.BACK, '.0' ];

            this._content.keypad( keypadSettings );
        },

        /* This method is called when the screen is about to be unloaded or when the developer removes the eXtension
         * from a field. Remove all DOM elements you have created and, to avoid memory leaks, delete any references
         * to them. Also remove any AXES event listeners you registered.
         */
        destroy : function()
        {
            if ( this._content )
            {
                this._content.keypad( 'destroy' );
                this._content.remove();
            }
            if ( AXES.isTS2Engine )
                this._field.removeAllListenersByContext( this );
            else
                this._field.removeListener( "ValueChanged", this._handleFieldValueChanged );
            delete this._content;
            delete this._field;
            delete this._fieldDiv;
        },

        _setValue : function()
        {
            /* Updates the display value to reflect the actual value of the field - unless the designer
            * has set the "value" property. In which case use that instead. */
            var field = this._field;
            val = this.getPropertyValue( "value" );
            var signedNum;
            if ( !( AXES.Utils.isString( val ) && val.length > 0 ) )
                val = this._field.getValue();

            switch ( field ? field.type : AXES.Terminal.Field.Types.INPUT )
            {
            case AXES.Terminal.Field.Types.OUTPUT:
                this._content.text( val );
                break;
            case AXES.Terminal.Field.Types.INPUT:
                /* Remove any padding from right aligned or signed number fields. Backup the 
                 * resulting value so changes can be detected later. */
                signedNum = ( ( ( field.fieldFormatWord & 0x700 ) >> 8 ) == 7 );
                this._masterVal = ( field.rightAlign || signedNum ) ? val.ltrim() : val.rtrim();
                if ( signedNum && ( " -".indexOf( this._masterVal.slice( -1 ) ) < 0 ) )
                    this._masterVal += " ";
                this._content.val( this._masterVal );
                break;

            default:
            }
        },

        /* Field events. These are events that are generated by the AXES.Terminal.Field
         * object.  The Axes event mechanism allows the context of the event
         * handler to be defined when it is registered.  So, unlike browser events,
         * the "this" keyword in these handlers refers to this extension object.
         */
        _setFocus : function( event )
        {
            // May not be able to set the focus if the field is invisible
            try
            {
                switch ( event.field.type )
                {
                case AXES.Terminal.Field.Types.INPUT:
                    this._content[ 0 ].focus();
                    break;

                default:
                }
            }
            catch( e ){}
        },

        _handleFieldValueChanged : function( event )
        {
            this._setValue();
        },

        _fieldAddedToDOM : function( event )
        {
            var $c = this._content;
            var newStyle = {};
            var size = this._field.getSize();
            // Extension elements size themselves. Only force the issue if the
            // designer has explicitly set a size.
            var s = this._fieldDiv.style;
            if ( s.width != "" )
                $c.width( size.width - ( $c.outerWidth() - $c.innerWidth() ) );
            if ( s.height != "" && !$c.is( "input" ) )
                $c.height( size.height - ( $c.outerHeight() - $c.innerHeight() ) );
        },

        /* Browser events on the field element.
         * In these event handlers "this" refers to the browser element that
         * received the event. Because of the way this extension is defined
         * within an object, we cannot use closures to get access to the
         * extension object.  To solve this the jQuery "bind" method is used
         * to assign the event handlers. This method allows a "data" property to
         * added to the event object. The "data" property is set to the extension.
         */
        _handleBlur : function( e )
        {
            /* The element has lost focus. Check if it has changed and, if so, save the value to the corresponding field.*/
            var self = e.data;
            var field = self._field, val = $( this ).val();
            if ( field.type == AXES.Terminal.Field.Types.INPUT && val != self._masterVal )
                field.setValue( val );
            if ( !self.isDesignMode() )
                self.raiseEvent( "onBlur" );
        },

        _handleFocus : function( e )
        {
            var self = e.data;
            var field = self._field;
            // Update the virtual terminal cursor position.
            if ( field.setCursor )
                field.setCursor();

            if ( self.isDesignMode() )
            {
                // Stop the focus event from being processed by the plugin. i.e. we don't want the keypad to
                // appear when in design mode.
                e.stopPropagation();
                e.stopImmediatePropagation();
            }
            else
                self.raiseEvent( "onFocus" );

            if ( !AXES.isTS2Engine )
            {
                $.keypad.mainDiv[ 0 ].style.position = "absolute";
                $.keypad.mainDiv[ 0 ].style.left = self._content[ 0 ].style.left;
            }
        }
    };

    /* Register with Axes */

    AXES.Extensions.add( extension );
})();

